<?php

    namespace App\Http\Controllers;

    use App\Models\Procedimento;
    use App\Exports\PainelExport;
    use Carbon\Carbon;
    use Illuminate\Http\Request;
    use Illuminate\Support\Facades\App;
    use Illuminate\Support\Facades\URL;
    use Maatwebsite\Excel\Facades\Excel;

    class AmbulatorialController extends Controller
    {
        /**
         * Create a new controller instance.
         *
         * @return void
         */
        public function __construct()
        {
            //
        }

        public function groupedData($uf)
        {
            $ufsg = '*';
            $ufmun = NULL;
            if ($uf != null && $uf != 'null') {
                $ufsg = substr($uf, 0, 2);
                $ufmun = substr($uf, 2, 6);
            }

            $terms = [];

            $terms[] = [
                'regexp' => ['PA_MVM' => date('Y').'[0-9]{2}']
            ];

            if ($ufmun && strlen($ufmun) == 6) {
                $terms[] = [
                    'term' => ['PA_UFMUN' => $ufmun]
                ];
            }

            $params = [
                'size' => 0,
                'query' => [
                    'bool' => [
                        'must' => $terms
                    ]
                ],
                'aggs' => [
                    "val" => [
                        "sum" => [
                            "field" => "PA_VALPRO"
                        ]
                    ],
                    "qtd" => [
                        "sum" => [
                            "field" => "PA_QTDPRO"
                        ]
                    ]
                ]
            ];

            $result = $this->fetchData('pa'.$ufsg, $params);

            $data = [
                'qtd' => $result['aggregations']['qtd']['value'],
                'val' => (float)number_format($result['aggregations']['val']['value'], 2, '.', ''),
            ];

            return response()->json($data, 200);
        }

        public function painelData($uf, Request $request)
        {
            setlocale(LC_ALL, 'pt_BR.utf8');

            $ufsg = '*';
            $ufmun = NULL;
            if ($uf != null && $uf != 'null') {
                $ufsg = substr($uf, 0, 2);
                $ufmun = substr($uf, 2, 6);
            }

            // recupera cache se o município não foi informado e nao foram feito filtros
            if (!$this->isValidMun($ufmun) && empty($request->toArray())) {
                $cached = \Cache::get("panel-data-{$ufsg}");

                // verifica execução em console para permitir que o comando gere cache
                if (!is_null($cached) && !app()->runningInConsole()) {
                    return response()->json($cached, 200);
                }
            }

            $dates = [];
            for ($i = date('Y'); $i > date('Y') - 10; $i--) {
                $dates[] = (string)$i;
            }

            $groupBy = 'y';
            if ($request->filled('periodo')) {
                $pCollection = collect($request->get('periodo'));

                $values = $pCollection->pluck('val');
                $dates = $pCollection->pluck('name');

                if (strlen($values[0]) == 6) {
                    $groupBy = 'm';
                }
            }

            $dateSize = (date('Y') - 2007) * 12;
            if ($groupBy == 'm') {
                $dateSize = 10;
            }

            $byYearTerms = [
                "exclude" => "PA.*",
                'size' => $dateSize,
                "order" => ["_key" => "desc"],
                'field' => 'PA_MVM.keyword'
            ];

            $query = $this->filterData($request, $ufmun);

            $params = [
                'size' => 0,
                'query' => $query,
                'aggs' => [
                    'by_year' => [
                        'terms' => $byYearTerms,
                        'aggs' => [
                            "valAprovado" => ["sum" => ["field" => "PA_VALAPR"]],
                            "valApresentado" => ["sum" => ["field" => "PA_VALPRO"]],
                            "qtdAprovado" => ["sum" => ["field" => "PA_QTDAPR"]],
                            "qtdApresentado" => ["sum" => ["field" => "PA_QTDPRO"]]
                        ]
                    ]
                ]
            ];

            $result = $this->fetchData('pa'.$ufsg, $params);

            $response = $result['aggregations']['by_year']['buckets'];

            $data = [];
            foreach ($dates as $date) {
                $data[$date] = [
                    'qtdAprovado' => 0,
                    'qtdApresentado' => 0,
                    'valAprovado' => 0,
                    'valApresentado' => 0,
                    'label' => $date
                ];
            }

            for ($i = 0; $i < count($response); $i++) {
                $date = $response[$i]['key'];

                if ($groupBy == 'm') {
                    $translate = [1 => 'Janeiro', 'Fevereiro', 'Março', 'Abril', 'Maio', 'Junho', 'Julho', 'Agosto', 'Setembro', 'Outubro', 'Novembro', 'Dezembro'];
                    $month = (int)Carbon::createFromFormat('Ym', $response[$i]['key'])->format('m');
                    $year = Carbon::createFromFormat('Ym', $response[$i]['key'])->format('Y');

                    $date = ucfirst($translate[$month]).' / '.$year;
                } else {
                    $date = substr($date, 0, 4);
                }

                if (isset($data[$date])) {
                    $qApro = $data[$date]['qtdAprovado'] + $response[$i]['qtdAprovado']['value'];
                    $qApre = $data[$date]['qtdApresentado'] + $response[$i]['qtdApresentado']['value'];
                    $vApro = $data[$date]['valAprovado'] + $response[$i]['valAprovado']['value'];
                    $vApre = $data[$date]['valApresentado'] + $response[$i]['valApresentado']['value'];

                    $data[$date] = [
                        'qtdAprovado' => $qApro,
                        'qtdApresentado' => $qApre,
                        'valAprovado' => $vApro,
                        'valApresentado' => $vApre,
                        'label' => $date
                    ];
                }
            }

            $data = array_values($data);

            // grava cache se o município não foi informado
            if (!$this->isValidMun($ufmun) && empty($request->toArray())) {
                \Cache::put("panel-data-{$ufsg}", $data);
            }

            return response()->json($data, 200);
        }

        public function getOptions($field)
        {
            $opts = [];
            if ($field == 'proc_ids') {
                $params = [
                    'size' => 5900,
                    '_source' => [
                        'val',
                        'text',
                        'tooltip'
                    ]
                ];

                $result = $this->fetchData('procedimentos', $params);

                $opts = $result['hits']['hits'];

                $opts = collect($opts)->pluck('_source');
            } else if ($field == 'estab') {
                $opts = [
                    [
                        'val' => 74,
                        'text' => "ACADEMIA DA SAÚDE                                  ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 81,
                        'text' => "CENTRAL DE REGULAÇÃO                              ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 76,
                        'text' => "CENTRAL DE REGULAÇÃO MÉDICA DAS URGÊNCIAS          ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 71,
                        'text' => "CENTRO DE APOIO A SAÚDE DA FAMÍLIA-CASF            ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 69,
                        'text' => "CENTRO DE ATENÇÃO HEMOTERÁPICA E/OU HEMATOLÓGICA   ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 70,
                        'text' => "CENTRO DE ATENÇÃO PSICOSSOCIAL-CAPS                ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 61,
                        'text' => "CENTRO DE PARTO NORMAL                             ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 2,
                        'text' => "CENTRO DE SAÚDE/UNIDADE BÁSICA DE SAÚDE            ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 64,
                        'text' => "CENTRAL DE REGULAÇÃO DE SERVIÇOS DE SAÚDE          ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 82,
                        'text' => "CENTRAL DE NOTIF,CAPTAÇÃO E DISTRIB DE ORGÃOS EST  ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 36,
                        'text' => "CLÍNICA ESPECIALIZADA/AMBULATORIO ESPECIALIZADO    ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 22,
                        'text' => "CONSULTÓRIO                                        ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 60,
                        'text' => "COOPERATIVA                                        ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 43,
                        'text' => "FARMÁCIA                                           ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 7,
                        'text' => "HOSPITAL ESPECIALIZADO                             ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 5,
                        'text' => "HOSPITAL GERAL                                     ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 62,
                        'text' => "HOSPITAL DIA                                       ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 67,
                        'text' => "LABORATÓRIO CENTRAL DE SAÚDE PÚBLICA - LACEN       ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 80,
                        'text' => "LABORATÓRIO DE SAÚDE PÚBLICA                              ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 4,
                        'text' => "POLICLÍNICA                                        ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 79,
                        'text' => "OFICINA ORTOPÉDICA                                 ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 1,
                        'text' => "POSTO DE SAÚDE                                     ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 73,
                        'text' => "PRONTO ATENDIMENTO                                 ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 21,
                        'text' => "PRONTO SOCORRO ESPECIALIZADO                       ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 20,
                        'text' => "PRONTO SOCORRO GERAL                               ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 68,
                        'text' => "SECRETARIA DE SAÚDE                                ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 77,
                        'text' => "SERVIÇO DE ATENÇÃO DOMICILIAR ISOLADO (HOME CARE)  ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 63,
                        'text' => "UNIDADE AUTORIZADORA                               ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 72,
                        'text' => "UNIDADE DE ATENÇÃO À SAÚDE INDÍGENA                ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 78,
                        'text' => "UNIDADE DE ATENÇÃO EM REGIME RESIDENCIAL           ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 39,
                        'text' => "UNIDADE DE SERVIÇO DE APOIO DE DIAGNOSE E TERAPIA  ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 45,
                        'text' => "UNIDADE DE SAÚDE DA FAMILIA                        ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 50,
                        'text' => "UNIDADE DE VIGILÂNCIA EM SAÚDE                     ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 65,
                        'text' => "UNIDADE DE VIGILÂNCIA EPIDEMIOLOGIA (ANTIGO)       ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 66,
                        'text' => "UNIDADE DE VIGILÂNCIA SANITARIA (ANTIGO)           ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 15,
                        'text' => "UNIDADE MISTA                                      ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 42,
                        'text' => "UNIDADE MOVEL DE NÍVEL PRE-HOSP-URGENCIA/EMERGÊNCIA",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 32,
                        'text' => "UNIDADE MOVEL FLUVIAL                              ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 40,
                        'text' => "UNIDADE MOVEL TERRESTRE                            ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 75,
                        'text' => "TELESAÚDE                                          ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 83,
                        'text' => "POLO DE PREVENÇÃO DE DOENÇAS E AGRAVOS E PROM DA SAÚDE",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 9,
                        'text' => "PRONTO SOCORRO DE HOSPITAL GERAL (ANTIGO)          ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => 12,
                        'text' => "PRONTO SOCORRO TRAUMATO-ORTOPÉDICO (ANTIGO)        ",
                        'tooltip' => ''
                    ],
                    [
                        'val' => -99,
                        'text' => "TIPO ESTABELECIMENTO NÃO INFORMADO                 ",
                        'tooltip' => ''
                    ]
                ];
            } else if ($field == 'cbo') {
                $params = [
                    'size' => 2800,
                    '_source' => [
                        'val',
                        'name'
                    ]
                ];

                $result = $this->fetchData('cbo', $params);

                $options = $result['hits']['hits'];

                $opts = collect($options)->pluck('_source')->map(function ($option) {
                    $option['tooltip'] = $option['name'];
                    $option['text'] = $option['val'];
                    unset($option['name']);

                    return $option;
                });
            } else if ($field == 'm_atend' || $field == 'm_resid') {
                $json = json_decode(file_get_contents(base_path('/public/esusgeo/municipio.json')), true);
                $opts = [];
                foreach ($json['municipio'] as $municipio) {
                    $tooltip = $municipio['uf'].' - '.$municipio['nome'];
                    $text = $municipio['cod'];

                    $opts[] = [
                        'text' => $text,
                        'val' => $municipio['cod'],
                        'tooltip' => $tooltip
                    ];
                }
            } else if ($field == 'periodo') {
                for ($i = (int)date('Y'); $i > 2007; $i--) {
                    $opts[] = $i;
                }
            } else if ($field == 'c_atend') {
                $opts = [
                    [
                        'val' => '1',
                        'text' => 'Eletivo',
                        'tooltip' => ''
                    ], [
                        'val' => '2',
                        'text' => 'Urgência',
                        'tooltip' => ''
                    ], [
                        'val' => '3',
                        'text' => 'Acidente no local trabalho ou a serviço da empresa',
                        'tooltip' => ''
                    ], [
                        'val' => '4',
                        'text' => 'Acidente no trajeto para o trabalho',
                        'tooltip' => ''
                    ], [
                        'val' => '5',
                        'text' => 'Outros tipos de acidente de trânsito',
                        'tooltip' => ''
                    ], [
                        'val' => '6',
                        'text' => 'Outros tipos lesões/envenenamentos (Agent.fis./quim.)',
                        'tooltip' => ''
                    ], [
                        'val' => '99',
                        'text' => 'Informação inexistente (BPA-C)',
                        'tooltip' => ''
                    ], [
                        'val' => '0',
                        'text' => 'Carater de atendimento não informado',
                        'tooltip' => ''
                    ]
                ];
            } else if ($field == 'complex') {
                $opts = [
                    [
                        'val' => '1',
                        'text' => 'Atenção Básica',
                        'tooltip' => ''
                    ], [
                        'val' => '2',
                        'text' => 'Média Complexidade',
                        'tooltip' => ''
                    ], [
                        'val' => '3',
                        'text' => 'Alta Complexidade',
                        'tooltip' => ''
                    ], [
                        'val' => '0',
                        'text' => 'Não se Aplica',
                        'tooltip' => ''
                    ]
                ];
            } else if ($field == 'laudos') {
                $opts = [
                    [
                        'val' => 1,
                        'text' => "Medicamentos",
                        'tooltip' => ''
                    ], [
                        'val' => 2,
                        'text' => "Nefrologia",
                        'tooltip' => ''
                    ], [
                        'val' => 3,
                        'text' => "Quimioterapia",
                        'tooltip' => ''
                    ], [
                        'val' => 4,
                        'text' => "Radioterapia",
                        'tooltip' => ''
                    ], [
                        'val' => 5,
                        'text' => "Bariátrica",
                        'tooltip' => ''
                    ], [
                        'val' => 6,
                        'text' => "Diversos",
                        'tooltip' => ''
                    ]
                ];
            } else if ($field == 'grupos') {
                $opts = [
                    [
                        'val' => '01',
                        'text' => '01 - Ações de promoção e prevenção em saúde',
                        'tooltip' => ''
                    ], [
                        'val' => '02',
                        'text' => '02 - Procedimentos com finalidade diagnóstica',
                        'tooltip' => ''
                    ], [
                        'val' => '03',
                        'text' => '03 - Procedimentos clínicos',
                        'tooltip' => ''
                    ], [
                        'val' => '04',
                        'text' => '04 - Procedimentos cirúrgicos',
                        'tooltip' => ''
                    ], [
                        'val' => '05',
                        'text' => '05 - Transplantes de órgãos, tecidos e células',
                        'tooltip' => ''
                    ], [
                        'val' => '06',
                        'text' => '06 - Medicamentos',
                        'tooltip' => ''
                    ], [
                        'val' => '07',
                        'text' => '07 - Órteses, próteses e materiais especiais',
                        'tooltip' => ''
                    ], [
                        'val' => '08',
                        'text' => '08 - Ações complementares da atenção à saúde',
                        'tooltip' => ''
                    ]
                ];
            }

            return response()->json($opts, 200);
        }

        public function getFiltros($uf, Request $request)
        {
            $ufsg = '*';
            $ufmun = NULL;
            if ($uf != null && $uf != 'null') {
                $ufsg = substr($uf, 0, 2);
                $ufmun = substr($uf, 2, 6);
            }

            $query = $this->filterData($request, $ufmun);

            $dateSize = 12 * (date('Y') - 2007);

            $params = [
                'size' => 0,
                'query' => $query,
                'aggs' => [
                    'periodo' => [
                        'terms' => [
                            'field' => 'PA_MVM.keyword',
                            "exclude" => "PA.*",
                            'size' => $dateSize,
                            "order" => ["_key" => "desc"]
                        ]
                    ],
                    'proc_ids' => [
                        'terms' => [
                            'field' => 'PA_PROC_ID.keyword',
                            "exclude" => "PA.*",
                            'size' => 5200,
                            "order" => ["_key" => "asc"]
                        ]
                    ],
                    'c_atend' => [
                        'terms' => [
                            'field' => 'PA_CATEND',
                            'size' => 15,
                            "order" => ["_key" => "asc"]
                        ]
                    ],
                    'm_atend' => [
                        'terms' => [
                            'field' => 'PA_UFMUN',
                            'size' => 6000,
                            "order" => ["_key" => "asc"]
                        ]
                    ],
                    'm_resid' => [
                        'terms' => [
                            'field' => 'PA_MUNPCN',
                            'size' => 6000,
                            "order" => ["_key" => "asc"]
                        ]
                    ],
                    'cbo' => [
                        'terms' => [
                            'field' => 'PA_CBOCOD',
                            'size' => 2800,
                            "order" => ["_key" => "asc"]
                        ]
                    ],
                    'estab' => [
                        'terms' => [
                            'field' => 'PA_TPUPS',
                            'size' => 50,
                            "order" => ["_key" => "asc"]
                        ]
                    ],
                    'complex' => [
                        'terms' => [
                            'field' => 'PA_NIVCPL',
                            'size' => 10,
                            "order" => ["_key" => "asc"]
                        ]
                    ]
                ]
            ];

            $result = $this->fetchData('pa'.$ufsg, $params);

            $result = $result['aggregations'];

            $filters = [];
            foreach ($result as $field => $values) {
                $options = collect($values['buckets'])->pluck('key');
                if ($field == 'periodo') {
                    $filters['periodo'] = $options->map(function ($item) {
                        return substr($item, 0, 4);
                    })->unique()->implode(',');
                } else {
                    if ($field == 'proc_ids') {
                        $filters['grupos'] = $options->map(function ($item) {
                            return substr($item, 0, 2);
                        })->unique()->implode(',');
                    }
                    $filters[$field] = $options->implode(',');
                }
            }

            $queryLaudos = $query;
            $queryLaudos['bool']['must'][] = [
                'terms' => [
                    'PA_DOCORIG.keyword' => ['P', 'S']
                ]
            ];

            $paramsLaudos = [
                'size' => 0,
                'query' => $queryLaudos,
                'aggs' => [
                    'medicamentos' => [
                        'terms' => [
                            'field' => 'PA_PROC_ID.keyword',
                            "include" => "06[0-9]{8}",
                            'size' => 1
                        ]
                    ],
                    'nefrologia' => [
                        'terms' => [
                            'field' => 'PA_PROC_ID.keyword',
                            "include" => "(030501[0-9]{4})|(05180[12][0-9]{4})",
                            'size' => 1
                        ]
                    ],
                    'quimioterapia' => [
                        'terms' => [
                            'field' => 'PA_PROC_ID.keyword',
                            "include" => "03040[2-8][0-9]{4}",
                            'size' => 1
                        ]
                    ],
                    'radioterapia' => [
                        'terms' => [
                            'field' => 'PA_PROC_ID.keyword',
                            "include" => "030401[0-9]{4}",
                            'size' => 1
                        ]
                    ],
                    'bariatrica' => [
                        'terms' => [
                            'field' => 'PA_PROC_ID.keyword',
                            "include" => "030112[0-9]{4}",
                            'size' => 1
                        ]
                    ],
                    'diversos' => [
                        'terms' => [
                            'field' => 'PA_PROC_ID.keyword',
                            "exclude" => "(06[0-9]{8})|(030501[0-9]{4})|(05180[12][0-9]{4})|(03040[2-8][0-9]{4})|(030401[0-9]{4})|(030112[0-9]{4})",
                            'size' => 1
                        ]
                    ]
                ]
            ];

            $laudos = [1 => 'medicamentos', 'nefrologia', 'quimioterapia', 'radioterapia', 'bariatrica', 'diversos'];

            $resultLaudos = $this->fetchData('pa'.$ufsg, $paramsLaudos);

            $resultLaudos = $resultLaudos['aggregations'];

            $filters['laudos'] = [];
            foreach ($resultLaudos as $field => $values) {
                $options = collect($values['buckets'])->pluck('key');

                $laudoIds = array_flip($laudos);
                $laudoVal = $laudoIds[$field];

                if (!empty($options->toArray())) {
                    $filters['laudos'][] = $laudoVal;
                }
            }

            $filters['laudos'] = implode(',', $filters['laudos']);

            return response()->json($filters, 200);
        }

        public function export($type, $uf, Request $request)
        {
            $filename = 'painel-'.date('YmdHis').'-'.rand(1000, 9999).'.'.$type;
            $filtros = $request->toArray();
            unset($filtros['name']);

            if ($type == 'pdf') {
                $data = $this->getFileSimpleData($uf, $request, $type);

                $pdf = App::make('dompdf.wrapper');
                $pdf->loadView('exports.painel', $data);
                $pdf->save(base_path('public/storage/'.$filename));
            } else if ($type == 'json') {
                $data = $this->getFileSimpleData($uf, $request, $type);

                unset($data['type']);

                foreach ($data['filtros'] as $filtro => $value) {
                    $data['filtros'][$filtro] = collect($value)->pluck('name')->implode('; ');
                }

                $data['legenda'] = str_replace(['ç', 'ã', 'õ'], ['c', 'a', 'o'], $data['legenda']);

                $handle = fopen(base_path('public/storage/'.$filename), 'w');
                fwrite($handle, json_encode($data, JSON_UNESCAPED_UNICODE));
                fclose($handle);
            } else if ($type == 'jpg' || $type == 'png') {
                $data = $this->painelData($uf, $request);
                $data = json_decode(json_encode($data->getData()), true);
                $graphData = $this->getGraphData($data, $request);
//                return view('exports.painel-graph', $graphData);

                $pdf = App::make('dompdf.wrapper');
                $pdf->loadView('exports.painel-graph', $graphData);
//                return $pdf->stream();

                $pdf->save(base_path('storage/temp-pdf-to-img.pdf'));

                $image = new \Imagick(base_path('storage/temp-pdf-to-img.pdf'));
                $image->setImageFormat($type);
                $image->resetIterator();
                $image->setBackgroundColor('#fff');
                $image->setCompressionQuality(\Imagick::COMPRESSION_NO);

                $combined = $image->appendImages(true);
//
//                header('Content-Type: image/png');
//                echo $combined;die;

                $combined->writeImage(base_path('public/storage/'.$filename));
                unlink(base_path('storage/temp-pdf-to-img.pdf'));
            } else if ($type == 'csv') {
                $detailed = [];
                $simple = [];
                if (!empty($filtros)) {
                    $detailed = $this->getFileDetailedData($uf, $request, $type);
                } else {
                    $simple = $this->getFileSimpleData($uf, $request, $type);
                }

                Excel::store(new PainelExport($simple, $detailed), 'public/'.$filename);

                // converte o arquivo de utf-8 para windows-1252
                $csv = file_get_contents(base_path('public/storage/'.$filename));
                $csv = mb_convert_encoding($csv, 'windows-1252', 'UTF-8');
                file_put_contents(base_path('public/storage/'.$filename), $csv);
            } else if ($type == 'xlsx') {
                $simpleData = $this->getFileSimpleData($uf, $request, $type);

                $detailedData = [];
                if (!empty($filtros)) {
                    $detailedData = $this->getFileDetailedData($uf, $request, $type);
                }


                Excel::store(new PainelExport($simpleData, $detailedData), 'public/'.$filename);
            }

            return response()->json(URL::to('storage/'.$filename), 200);
        }

        public function getFieldName($field, $values)
        {
            $name = $values['name'];
            if ($field == 'procedimento') {
                $name = $values['code'].' - '.$values['name'];
            }

            return $name;
        }

        private function isValidMun($ufmun)
        {
            return $ufmun && strlen($ufmun) == 6;
        }

        public function filterData($request, $ufmun)
        {
            $terms = [];

            if ($this->isValidMun($ufmun)) {
                $terms[] = [
                    'term' => ['PA_UFMUN' => $ufmun]
                ];
            }

            if ($request->filled("periodo")) {
                $values = collect($request->get('periodo'))->pluck('val');

                if (strlen($values[0]) == 6) {
                    $terms[] = [
                        'terms' => ['PA_MVM' => $values->toArray()]
                    ];
                } else {
                    $boolShould = [];
                    foreach ($values as $value) {
                        $boolShould[] = [
                            'regexp' => ['PA_MVM' => $value."[0-9]{2}"]
                        ];
                    }

                    $terms[] = [
                        'bool' => [
                            'should' => $boolShould
                        ]
                    ];
                }
            }

            if ($request->filled("c_atend")) {
                $values = collect($request->get('c_atend'))->pluck('val');

                $terms[] = [
                    'terms' => ['PA_CATEND' => $values->toArray()]
                ];
            }

            if ($request->filled("proc_ids")) {
                $values = collect($request->get('proc_ids'))->pluck('val');

                $terms[] = [
                    'terms' => ['PA_PROC_ID' => $values->toArray()]
                ];
            }

            if ($request->filled("m_atend")) {
                $values = collect($request->get('m_atend'))->pluck('val');

                $terms[] = [
                    'terms' => ['PA_UFMUN' => $values->toArray()]
                ];
            }

            if ($request->filled("m_resid")) {
                $values = collect($request->get('m_resid'))->pluck('val');

                $terms[] = [
                    'terms' => ['PA_MUNPCN' => $values->toArray()]
                ];
            }

            if ($request->filled("complex")) {
                $values = collect($request->get('complex'))->pluck('val');

                $terms[] = [
                    'terms' => ['PA_NIVCPL' => $values->toArray()]
                ];
            }

            if ($request->filled("cbo")) {
                $values = collect($request->get('cbo'))->pluck('val');

                $terms[] = [
                    'terms' => ['PA_CBOCOD' => $values->toArray()]
                ];
            }

            if ($request->filled("estab")) {
                $values = collect($request->get('estab'))->pluck('val');

                $terms[] = [
                    'terms' => ['PA_TPUPS' => $values->toArray()]
                ];
            }

            if ($request->filled("laudos")) {
                $values = collect($request->get('laudos'))->pluck('val');

                $laudos = $this->getLaudosFiltros($values);

                $terms[] = [
                    'terms' => ['PA_DOCORIG.keyword' => ['P', 'S']]
                ];

                $terms[] = $laudos;
            }

            if ($request->filled("grupos")) {
                $values = collect($request->get('grupos'))->pluck('val');

                $boolShould = [];
                foreach ($values as $value) {
                    $boolShould[] = [
                        'regexp' => ['PA_PROC_ID' => $value."[0-9]{8}"]
                    ];
                }

                $terms[] = [
                    'bool' => [
                        'should' => $boolShould
                    ]
                ];
            }

            return [
                'bool' => [
                    'must' => $terms
                ]
            ];
        }

        public function getLaudosFiltros($values)
        {
            /*
             * Identifica os laudos através dos procedimentos
             * Procedimento contem 10 caracteres numéricos e é dividido em 5 grupos na seguinte ordem:
             * - Grupo: 2 caracteres
             * - Subgrupo: 2 caracteres
             * - Forma de Organização: 2 caracteres
             * - Procedimento: 3 caracteres
             * - Digito Verificador: 1 caracter
             *
             * Só precisamos os procedimentos APAC, então filtramos
             *  a origem (DOCORIG) para APAC Principal (P) e APAC Secundária (S)
             */

            $boolShould = [];

            if ($values->contains('1')) { // Medicamentos
                // Grupo: 06
                $boolShould[] = [
                    'regexp' => ['PA_PROC_ID' => '06[0-9]{8}']
                ];
            }

            if ($values->contains('2')) { // Nefrologia
                // Grupo: 03
                // Subgrupo: 05
                // Forma de organização: 01
                $boolShould[] = [
                    'regexp' => ['PA_PROC_ID' => '030501[0-9]{4}']
                ];

                // Grupo: 05
                // Subgrupo: 18
                // Forma de organização: 01 e 02
                $boolShould[] = [
                    'regexp' => ['PA_PROC_ID' => '05180[12][0-9]{4}']
                ];
            }

            if ($values->contains('3')) { // Quimioterapia
                // Grupo: 03
                // Subgrupo: 04
                // Forma de organização: 02 - 08
                $boolShould[] = [
                    'regexp' => ['PA_PROC_ID' => '03040[2-8][0-9]{4}']
                ];
            }

            if ($values->contains('4')) { // Radioterapia
                // Grupo: 03
                // Subgrupo: 04
                // Forma de organização: 01
                $boolShould[] = [
                    'regexp' => ['PA_PROC_ID' => '030401[0-9]{4}']
                ];
            }

            if ($values->contains('5')) { // Bariátrica
                // Grupo: 03
                // Subgrupo: 01
                // Forma de organização: 12
                $boolShould[] = [
                    'regexp' => ['PA_PROC_ID' => '030112[0-9]{4}']
                ];
            }

            if ($values->contains('6')) { // Diversos
                // Tudo o que NÃO for os filtros de cima
                $boolShould[] = [
                    'bool' => [
                        'must_not' => [
                            0 => ['regexp' => ['PA_PROC_ID' => '06[0-9]{8}']], // Medicamentos
                            1 => ['regexp' => ['PA_PROC_ID' => '030501[0-9]{4}']], // Nefrologia 1
                            2 => ['regexp' => ['PA_PROC_ID' => '05180[12][0-9]{4}']], // Nefrologia 2
                            3 => ['regexp' => ['PA_PROC_ID' => '03040[2-8][0-9]{4}']], // Quimioterapia
                            4 => ['regexp' => ['PA_PROC_ID' => '030401[0-9]{4}']], // Radioterapia
                            5 => ['regexp' => ['PA_PROC_ID' => '030112[0-9]{4}']], // Bariátrica
                        ]
                    ]
                ];
            }

            return [
                'bool' => [
                    'should' => $boolShould
                ]
            ];
        }

        public function procedimentos()
        {
            $procs = Procedimento::selectRaw('`code` as val, `code` as text, CONCAT(UCASE(LEFT(name, 1)), LCASE(SUBSTRING(name, 2))) as tooltip')->get();

            return response()->json($procs, 200);
        }

        public function getLastMonth($ajax = true)
        {
            $params = [
                'size' => 1,
                '_source' => [
                    'PA_MVM'
                ],
                'sort' => [
                    ['PA_MVM.keyword' => 'desc']
                ],
                'query' => [
                    'bool' => [
                        'must' => [
                            0 => [
                                'regexp' => [
                                    'PA_MVM' => '[0-9]{6}'
                                ]
                            ]
                        ]
                    ]
                ]
            ];

            $data = $this->fetchData('paac', $params);

            $date = $data['hits']['hits'][0]['_source']['PA_MVM'];

            $date = Carbon::createFromFormat('Ym', $date)->format('m/Y');

            if ($ajax) {
                return response()->json(['lastUpdated' => $date], 200);
            } else {
                return $date;
            }
        }

        public function geo(Request $request)
        {
            $modelo = $request->get('modelo') == 'amazonialegal,semiarido,sisfronteiras,qualisus' ? 'diversos' : $request->get('modelo');
            $json = json_decode(file_get_contents(base_path('/public/esusgeo/'.$modelo.'.json')));

            return response()->json($json, 200);
        }

        public function getFileSimpleData($uf, $request, $type)
        {
            setlocale(LC_TIME, 'pt_BR.utf8');

            $dates = [];
            for ($i = date('Y'); $i > date('Y') - 10; $i--) {
                $dates[] = (string)$i;
            }

            $ufsg = '*';
            $ufmun = NULL;
            if ($uf != null && $uf != 'null') {
                $ufsg = substr($uf, 0, 2);
                $ufmun = substr($uf, 2, 6);
            }

            $groupBy = 'y';
            if ($request->filled('periodo')) {
                $pCollection = collect($request->get('periodo'));

                $values = $pCollection->pluck('val');
                $dates = $pCollection->pluck('name');

                if (strlen($values[0]) == 6) {
                    $groupBy = 'm';
                }
            }

            $dateSize = (date('Y') - 2007) * 12;

            $byYearTerms = [
                "exclude" => "PA_M*",
                'field' => "PA_MVM.keyword",
                'size' => $dateSize,
                "order" => ["_key" => "desc"]
            ];

            $query = $this->filterData($request, $ufmun);

            $params = [
                'size' => 0,
                'query' => $query,
                'aggs' => [
                    'by_year' => [
                        'terms' => $byYearTerms,
                        'aggs' => [
                            "valAprovado" => ["sum" => ["field" => "PA_VALAPR"]],
                            "valApresentado" => ["sum" => ["field" => "PA_VALPRO"]],
                            "qtdAprovado" => ["sum" => ["field" => "PA_QTDAPR"]],
                            "qtdApresentado" => ["sum" => ["field" => "PA_QTDPRO"]]
                        ]
                    ]
                ]
            ];

            $result = $this->fetchData('pa'.$ufsg, $params);

            $elasticData = $result['aggregations']['by_year']['buckets'];

            $data = [];
            foreach ($dates as $date) {
                $data[$date] = [
                    'qtdAprovado' => 0,
                    'qtdApresentado' => 0,
                    'valAprovado' => 0,
                    'valApresentado' => 0,
                    'label' => $date
                ];
            }

            for ($i = 0; $i < count($elasticData); $i++) {
                $date = $elasticData[$i]['key'];

                if ($groupBy == 'm') {
                    $translate = [1 => 'Janeiro', 'Fevereiro', 'Março', 'Abril', 'Maio', 'Junho', 'Julho', 'Agosto', 'Setembro', 'Outubro', 'Novembro', 'Dezembro'];
                    $month = (int)Carbon::createFromFormat('Ym', $elasticData[$i]['key'])->format('m');
                    $year = Carbon::createFromFormat('Ym', $elasticData[$i]['key'])->format('Y');

                    $date = ucfirst($translate[$month]).' / '.$year;
                } else {
                    $date = substr($date, 0, 4);
                }

                if (isset($data[$date])) {
                    $qApro = $data[$date]['qtdAprovado'] + $elasticData[$i]['qtdAprovado']['value'];
                    $qApre = $data[$date]['qtdApresentado'] + $elasticData[$i]['qtdApresentado']['value'];
                    $vApro = $data[$date]['valAprovado'] + $elasticData[$i]['valAprovado']['value'];
                    $vApre = $data[$date]['valApresentado'] + $elasticData[$i]['valApresentado']['value'];

                    $data[$date] = [
                        'qtdAprovado' => $qApro,
                        'qtdApresentado' => $qApre,
                        'valAprovado' => $vApro,
                        'valApresentado' => $vApre,
                        'label' => $date
                    ];
                }
            }

            $data = array_values($data);

            $filtros = $request->toArray();
            unset($filtros['name']);

            $lastUpdated = $this->getLastMonth(false);

            return [
                'nome' => 'Produção ambulatorial - '.$request->get('name'),
                'filtros' => $filtros,
                'painel' => $data,
                'type' => $type,
                "legenda" => "Fonte de dados: Sistema de Informações Ambulatoriais - SIASUS. Extração em ".date('d/m/Y à\s H:i\h').". Dados atualizados em ".$lastUpdated
            ];
        }

        public function getFileDetailedData($uf, $request, $type)
        {
            setlocale(LC_TIME, 'pt_BR.utf8');

            $dates = [];
            for ($i = date('Y'); $i > date('Y') - 10; $i--) {
                $dates[] = (string)$i;
            }

            $ufsg = '*';
            $ufmun = NULL;
            if ($uf != null && $uf != 'null') {
                $ufsg = substr($uf, 0, 2);
                $ufmun = substr($uf, 2, 6);
            }

            $filtros = $request->toArray();
            unset($filtros['name']);
            array_multisort(array_map('count', $filtros), SORT_ASC, $filtros);
            if (!isset($filtros['periodo'])) {
                $filtros['periodo'] = [];
                for ($i = date('Y'); $i > date('Y') - 10; $i--) {
                    $filtros['periodo'][] = [
                        'name' => (string)$i,
                        'tooltip' => ''
                    ];
                }
            }

            $aggs = $this->getAggregations($filtros);

            $query = $this->filterData($request, $ufmun);

            $params = [
                'size' => 0,
                'query' => $query,
                'aggs' => $aggs
            ];

            $result = $this->fetchData('pa'.$ufsg, $params);

            $flattened = $this->flatArray($result['aggregations'], $filtros);

            $grouped = $this->groupArray($flattened);

            $lastUpdated = $this->getLastMonth(false);

            return [
                'nome' => 'Produção ambulatorial - '.$request->get('name'),
                'filtros' => $filtros,
                'painel' => $grouped,
                'type' => $type,
                "legenda" => "Fonte de dados: Sistema de Informações Ambulatoriais - SIASUS. Extração em ".date('d/m/Y à\s H:i\h').". Dados atualizados em ".$lastUpdated
            ];
        }

        public function groupArray($array)
        {
            $grouped = [];
            foreach ($array as $item) {
                $key = '';
                foreach ($item as $campo => $valor) {
                    if (!in_array($campo, ['qtdProduzido', 'qtdAprovado', 'valProduzido', 'valAprovado'])) {
                        $key .= $valor;
                    }
                }

                if (isset($grouped[$key])) {
                    $grouped[$key]['qtdProduzido'] = $grouped[$key]['qtdProduzido'] + $item['qtdProduzido'];
                    $grouped[$key]['qtdAprovado'] = $grouped[$key]['qtdAprovado'] + $item['qtdAprovado'];
                    $grouped[$key]['valProduzido'] = $grouped[$key]['valProduzido'] + $item['valProduzido'];
                    $grouped[$key]['valAprovado'] = $grouped[$key]['valAprovado'] + $item['valAprovado'];
                } else {
                    $grouped[$key] = $item;
                }
            }

            $grouped = array_values($grouped);

            for ($i = 0; $i < count($grouped); $i++) {
                $grouped[$i]['qtdProduzido'] = number_format($grouped[$i]['qtdProduzido'], 2, ',', '.');
                $grouped[$i]['qtdAprovado'] = number_format($grouped[$i]['qtdAprovado'], 2, ',', '.');
                $grouped[$i]['valProduzido'] = "R$ ".number_format($grouped[$i]['valProduzido'], 2, ',', '.');
                $grouped[$i]['valAprovado'] = "R$ ".number_format($grouped[$i]['valAprovado'], 2, ',', '.');
            }

            return $grouped;
        }

        public function getAggregations($filtros)
        {
            $aggs = [
                "qtdProduzido" => ["sum" => ["field" => "PA_QTDPRO"]],
                "qtdAprovado" => ["sum" => ["field" => "PA_QTDAPR"]],
                "valProduzido" => ["sum" => ["field" => "PA_VALPRO"]],
                "valAprovado" => ["sum" => ["field" => "PA_VALAPR"]],
            ];

            foreach ($filtros as $filtro => $values) {
                $selectedOptions = collect($values)->pluck('val');

                if ($filtro == 'laudos') {
                    $laudos = [
                        1 => [
                            'field' => 'PA_PROC_ID.keyword',
                            "include" => "06[0-9]{8}",
                            'size' => 5200
                        ],
                        2 => [
                            'field' => 'PA_PROC_ID.keyword',
                            "include" => "(030501[0-9]{4})|(05180[12][0-9]{4})",
                            'size' => 5200
                        ],
                        3 => [
                            'field' => 'PA_PROC_ID.keyword',
                            "include" => "03040[2-8][0-9]{4}",
                            'size' => 5200
                        ],
                        4 => [
                            'field' => 'PA_PROC_ID.keyword',
                            "include" => "030401[0-9]{4}",
                            'size' => 5200
                        ],
                        5 => [
                            'field' => 'PA_PROC_ID.keyword',
                            "include" => "030112[0-9]{4}",
                            'size' => 5200
                        ],
                        6 => [
                            'field' => 'PA_PROC_ID.keyword',
                            "exclude" => "(06[0-9]{8})|(030501[0-9]{4})|(05180[12][0-9]{4})|(03040[2-8][0-9]{4})|(030401[0-9]{4})|(030112[0-9]{4})",
                            'size' => 5200
                        ]
                    ];

                    $laudoAggs = [];
                    foreach ($values as $selected) {
                        $laudo = $selected['name'];
                        $val = $selected['val'];
                        $term = $laudos[$val];

                        $laudoAggs[$laudo] = [
                            'terms' => $term,
                            'aggs' => $aggs
                        ];
                    }

                    $aggs = $laudoAggs;
                } else {
                    $aggsTerms = [];
                    $translateField = [
                        'periodo' => "PA_MVM.keyword",
                        'grupos' => ["inline" => "doc['PA_PROC_ID.keyword'].getValue().substring(0,2)"],
                        'proc_ids' => "PA_PROC_ID.keyword",
                        'c_atend' => "PA_CATEND",
                        'm_atend' => "PA_UFMUN",
                        'm_resid' => "PA_MUNPCN",
                        'complex' => "PA_NIVCPL",
                        'estab' => "PA_TPUPS",
                        'cbo' => "PA_CBOCOD",
                    ];

                    $field = $translateField[$filtro];

                    $order = $filtro == 'periodo' ? 'desc' : 'asc';

                    $fieldType = 'field';
                    if ($filtro == 'grupos') {
                        $fieldType = 'script';
                    }

                    if ($filtro == 'periodo' && strlen($selectedOptions[0]) != 6) {
                        $fieldType = 'script';
                        $field = ["inline" => "doc['PA_MVM.keyword'].getValue().substring(0,4)"];
                    }

                    if (!in_array($filtro, ['c_atend', 'm_resid', 'm_atend', 'cbo', 'estab', 'complex'])) {
                        $aggsTerms['exclude'] = "PA.*";
                    }

                    $aggsTerms[$fieldType] = $field;
                    $aggsTerms['size'] = count($values);
                    $aggsTerms['order'] = ["_key" => $order];

                    $aggs = [
                        $filtro => [
                            'terms' => $aggsTerms,
                            'aggs' => $aggs
                        ]
                    ];
                }
            }

            return $aggs;
        }

        public function flatArray($array, $filtros)
        {
            $flat = [];

            foreach ($array as $field => $dados) {
                $index = 0;

                foreach ($dados['buckets'] as $grouped) {
                    end($grouped);
                    $lastKey = key($grouped);

                    if (in_array($lastKey, ['valProduzido', 'valAprovado'])) {
                        foreach ($grouped as $name => $value) {

                            if ($name != 'doc_count') {
                                $arrayName = $name == 'key' ? $field : $name;

                                if (is_array($value)) {
                                    $arrayValue = $value['value'];
                                } else {
                                    $arrayValue = $this->translateFileValue($field, $value, $filtros);
                                }

                                $flat[$index][$arrayName] = $arrayValue;
                            }
                        }
                        $index++;
                    } else {
                        $data = array_slice($grouped, 2);

                        foreach ($data as $campo => $values) {
                            $subArray = $this->flatArray([$campo => $values], $filtros);

                            foreach ($subArray as $item) {
                                $name = $field;
                                if (in_array($field, ['Diversos', 'Medicamentos', 'Nefrologia', 'Quimioterapia', 'Radiologia', 'Bariátrica'])) {
                                    $name = 'laudos';
                                }

                                $fieldVal = $this->translateFileValue($field, $grouped['key'], $filtros);

                                if (!empty($fieldVal)) {
                                    $arr = array_merge([$name => $fieldVal], $item);

                                    $flat[] = $arr;
                                }
                            }
                        }
                    }
                }
            }

            return $flat;
        }

        public function translateFileValue($field, $value, $filtros)
        {
            if (isset($filtros[$field])) {
                // pega as informações filtradas do valor atual
                $items = collect($filtros[$field]);
                if (isset($items[0]['val'])) {
                    $data = $items->keyBy('val');
                } else {
                    $data = $items->keyBy('name');
                }

                if (isset($data[$value])) {
                    $data = $data[$value];

                    // se houver tooltip, concatena ela no final da string
                    $tooltip = '';
                    if (!empty($data['tooltip'])) {
                        $tooltip = ' - '.$data['tooltip'];
                    }

                    // o campo traduzido deve ser o mesmo nome que é mostrado nas opções dos filtros
                    $translated = $data['name'].$tooltip;
                } else {
                    $translated = '';
                }
            } else {
                $translated = $field;
            }

            return $translated;
        }

        public function getGraphData($data, $request)
        {
            $graph = collect($data);

            $maxQtd = $graph->max('qtdApresentado');
            $maxVal = $graph->max('valApresentado');

            $yAxisShort = strlen($graph[0]['label']) == 4;
            $lastUpdated = $this->getLastMonth(false);

            $graph->transform(function ($item) use ($maxVal, $maxQtd) {
                foreach ($item as $name => $value) {
                    if ($name != 'label') {
                        if (strpos($name, 'qtd') !== false) {
                            $max = $maxQtd;

                            $valor = number_format($value, 0, ',', '');
                        } else {
                            $max = $maxVal;

                            $valor = $this->abbreviateNumbers($value, 2);
                        }

                        $width = ($value / $max) * 100;
                        $width = number_format($width, 2, '.', '');

                        $class = $width > 15 ? 'inside' : 'outside';

                        $item[$name] = [
                            'width' => $width,
                            'value' => $valor,
                            'class' => $class,
                        ];
                    }
                }

                return $item;
            });

            $filtros = $request->toArray();
            unset($filtros['name']);
            $graph = $graph->slice(0, 10);

            return [
                'nome' => 'Produção ambulatorial - '.$request->get('name'),
                'filtros' => $filtros,
                'graph' => $graph,
                'yAxisShort' => $yAxisShort,
                'lastUpdated' => $lastUpdated,
                "legenda" => "Fonte de dados: Sistema de Informações Ambulatoriais - SIASUS. Extração em ".date('d/m/Y à\s H:i\h')."."
            ];
        }

        public function abbreviateNumbers($n, $precision = 1)
        {
            if ($n < 999) {
                // 0 - 900
                $n_format = number_format($n, $precision);
                $suffix = '';
            } else if ($n < 999999) {
                // 0.9k-850k
                $n_format = number_format($n / 1000, $precision);
                $suffix = ' mil';
            } else if ($n < 999999999) {
                // 0.9m-850m
                $n_format = number_format($n / 1000000, $precision);
                $suffix = ' mi';
            } else if ($n < 999999999999) {
                // 0.9b-850b
                $n_format = number_format($n / 1000000000, $precision);
                $suffix = ' bi';
            } else {
                // 0.9t+
                $n_format = number_format($n / 1000000000000, $precision);
                $suffix = ' tri';
            }
            // Remove unecessary zeroes after decimal. "1.0" -> "1"; "1.00" -> "1"
            // Intentionally does not affect partials, eg "1.50" -> "1.50"
            if ($precision > 0) {
                $dotzero = '.'.str_repeat('0', $precision);
                $n_format = str_replace($dotzero, '', $n_format);
            }

            return $n_format.$suffix;
        }

        public function fetchData($index, $params)
        {
            $ch = curl_init();

            $url = "http://".env('ELASTIC_KEY').":9200/".strtolower($index)."/_search";

            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));

            //return the transfer as a string
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

            // $output contains the output string
            $output = curl_exec($ch);
            $result = json_decode($output, true);

            // close curl resource to free up system resources
            curl_close($ch);

            return $result;
        }
    }

